<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 04/04/2009 *
 * ===================                          Last change: 04/04/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : wrapper-functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Wrapper functions                                *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Wrapper-Funktionen                               *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2011 by Mailer Developer Team                   *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Read a given file
function readFromFile ($FQFN) {
	// Sanity-check if file is there (should be there, but just to make it sure)
	if (!isFileReadable($FQFN)) {
		// This should not happen
		debug_report_bug(__FUNCTION__, __LINE__, 'File ' . basename($FQFN) . ' is not readable!');
	} elseif (!isset($GLOBALS['file_content'][$FQFN])) {
		// Load the file
		if (function_exists('file_get_contents')) {
			// Use new function
			$GLOBALS['file_content'][$FQFN] = file_get_contents($FQFN);
		} else {
			// Fall-back to implode-file chain
			$GLOBALS['file_content'][$FQFN] = implode('', file($FQFN));
		}
	} // END - if

	// Return the content
	return $GLOBALS['file_content'][$FQFN];
}

// Writes content to a file
function writeToFile ($FQFN, $content, $aquireLock = false) {
	// Is the file writeable?
	if ((isFileReadable($FQFN)) && (!is_writeable($FQFN)) && (!changeMode($FQFN, 0644))) {
		// Not writeable!
		logDebugMessage(__FUNCTION__, __LINE__, sprintf("File %s not writeable.", basename($FQFN)));

		// Failed! :(
		return false;
	} // END - if

	// By default all is failed...
	$GLOBALS['file_readable'][$FQFN] = false;
	unset($GLOBALS['file_content'][$FQFN]);
	$return = false;

	// Is the function there?
	if (function_exists('file_put_contents')) {
		// With lock?
		if ($aquireLock === true) {
			// Write it directly with lock
			$return = file_put_contents($FQFN, $content, LOCK_EX);
		} else {
			// Write it directly
			$return = file_put_contents($FQFN, $content);
		}
	} else {
		// Write it with fopen
		$fp = fopen($FQFN, 'w') or debug_report_bug(__FUNCTION__, __LINE__, 'Cannot write to file ' . basename($FQFN) . '!');

		// Do we need to aquire a lock?
		if ($aquireLock === true) {
			// Aquire lock
			flock($fp, LOCK_EX);
		} // END - if

		// Write content
		$return = fwrite($fp, $content);

		// Close stream
		fclose($fp);
	}

	// Was something written?
	if ($return !== false) {
		// Mark it as readable
		$GLOBALS['file_readable'][$FQFN] = true;

		// Remember content in cache
		$GLOBALS['file_content'][$FQFN] = $content;
	} // END - if

	// Return status
	return (($return !== false) && (changeMode($FQFN, 0644)));
}

// Clears the output buffer. This function does *NOT* backup sent content.
function clearOutputBuffer () {
	// Trigger an error on failure
	if ((ob_get_length() > 0) && (!ob_end_clean())) {
		// Failed!
		debug_report_bug(__FUNCTION__, __LINE__, 'Failed to clean output buffer.');
	} // END - if
}

// Encode strings
// @TODO Implement $compress
function encodeString ($str, $compress = true) {
	$str = urlencode(base64_encode(compileUriCode($str)));
	return $str;
}

// Decode strings encoded with encodeString()
// @TODO Implement $decompress
function decodeString ($str, $decompress = true) {
	$str = compileUriCode(base64_decode(urldecode(compileUriCode($str))));
	return $str;
}

// Decode entities in a nicer way
function decodeEntities ($str, $quote = ENT_NOQUOTES) {
	// Decode the entities to UTF-8 now
	$decodedString = html_entity_decode($str, $quote, 'UTF-8');

	// Return decoded string
	return $decodedString;
}

// Merges an array together but only if both are arrays
function merge_array ($array1, $array2) {
	// Are both an array?
	if ((!is_array($array1)) && (!is_array($array2))) {
		// Both are not arrays
		debug_report_bug(__FUNCTION__, __LINE__, 'No arrays provided!');
	} elseif (!is_array($array1)) {
		// Left one is not an array
		debug_report_bug(__FUNCTION__, __LINE__, sprintf("array1 is not an array. array != %s", gettype($array1)));
	} elseif (!is_array($array2)) {
		// Right one is not an array
		debug_report_bug(__FUNCTION__, __LINE__, sprintf("array2 is not an array. array != %s", gettype($array2)));
	}

	// Merge all together
	return array_merge($array1, $array2);
}

// Check if given FQFN is a readable file
function isFileReadable ($FQFN) {
	// Do we have cache?
	if (!isset($GLOBALS['file_readable'][$FQFN])) {
		// Check all...
		$GLOBALS['file_readable'][$FQFN] = ((file_exists($FQFN)) && (is_file($FQFN)) && (is_readable($FQFN)));
	} // END - if

	// Return result
	return $GLOBALS['file_readable'][$FQFN];
}

// Checks wether the given FQFN is a directory and not ., .. or .svn
function isDirectory ($FQFN) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$FQFN])) {
		// Generate baseName
		$baseName = basename($FQFN);

		// Check it
		$GLOBALS[__FUNCTION__][$FQFN] = ((is_dir($FQFN)) && ($baseName != '.') && ($baseName != '..') && ($baseName != '.svn'));
	} // END - if

	// Return the result
	return $GLOBALS[__FUNCTION__][$FQFN];
}

// "Getter" for the real remote IP number
function detectRealIpAddress () {
	// Get remote ip from environment
	$remoteAddr = determineRealRemoteAddress();

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteAddr = getAnonymousRemoteAddress($remoteAddr);
	} // END - if

	// Return it
	return $remoteAddr;
}

// "Getter" for remote IP number
function detectRemoteAddr () {
	// Get remote ip from environment
	$remoteAddr = determineRealRemoteAddress(true);

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteAddr = getAnonymousRemoteAddress($remoteAddr);
	} // END - if

	// Return it
	return $remoteAddr;
}

// "Getter" for remote hostname
function detectRemoteHostname () {
	// Get remote ip from environment
	$remoteHost = getenv('REMOTE_HOST');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteHost = getAnonymousRemoteHost($remoteHost);
	} // END - if

	// Return it
	return $remoteHost;
}

// "Getter" for user agent
function detectUserAgent ($alwaysReal = false) {
	// Get remote ip from environment
	$userAgent = getenv('HTTP_USER_AGENT');

	// Is removeip installed?
	if ((isExtensionActive('removeip')) && ($alwaysReal === false)) {
		// Then anonymize it
		$userAgent = getAnonymousUserAgent($userAgent);
	} // END - if

	// Return it
	return $userAgent;
}

// "Getter" for referer
function detectReferer () {
	// Get remote ip from environment
	$referer = getenv('HTTP_REFERER');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$referer = getAnonymousReferer($referer);
	} // END - if

	// Return it
	return $referer;
}

// "Getter" for request URI
function detectRequestUri () {
	// Return it
	return (getenv('REQUEST_URI'));
}

// "Getter" for query string
function detectQueryString () {
	return str_replace('&', '&amp;', (getenv('QUERY_STRING')));
}

// "Getter" for SERVER_NAME
function detectServerName () {
	// Return it
	return (getenv('SERVER_NAME'));
}

// Removes any  existing www. from SERVER_NAME. This is very silly but enough
// for our purpose here.
function detectDomainName () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Get server name
		$domainName = detectServerName();

		// Is there any www. ?
		if (substr($domainName, 0, 4) == 'www.') {
			// Remove it
			$domainName = substr($domainName, 4);
		} // END - if

		// Set cache
		$GLOBALS[__FUNCTION__] = $domainName;
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Check wether we are installing
function isInstalling () {
	// Determine wether we are installing
	if (!isset($GLOBALS['mailer_installing'])) {
		// Check URL (css.php/js.php need this)
		$GLOBALS['mailer_installing'] = isGetRequestParameterSet('installing');
	} // END - if

	// Return result
	return $GLOBALS['mailer_installing'];
}

// Check wether this script is installed
function isInstalled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine wether this script is installed
		$GLOBALS[__FUNCTION__] = (
		(
			// First is config
			(
				(
					isConfigEntrySet('MXCHANGE_INSTALLED')
				) && (
					getConfig('MXCHANGE_INSTALLED') == 'Y'
				)
			)
		) || (
			// New config file found and loaded
			isIncludeReadable(getCachePath() . 'config-local.php')
		) || (
			(
				// New config file found, but not yet read
				isIncludeReadable(getCachePath() . 'config-local.php')
			) && (
				(
					// Only new config file is found
					!isIncludeReadable('inc/config.php')
				) || (
					// Is installation mode
					!isInstalling()
				)
			)
		));
	} // END - if

	// Then use the cache
	return $GLOBALS[__FUNCTION__];
}

// Check wether an admin is registered
function isAdminRegistered () {
	// Is cache set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Simply check it
		$GLOBALS[__FUNCTION__] = ((isConfigEntrySet('ADMIN_REGISTERED')) && (getConfig('ADMIN_REGISTERED') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__];
}

// Checks wether the reset mode is active
function isResetModeEnabled () {
	// Now simply check it
	return ((isset($GLOBALS['reset_enabled'])) && ($GLOBALS['reset_enabled'] === true));
}

// Checks wether the debug mode is enabled
function isDebugModeEnabled () {
	// Is cache set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Simply check it
		$GLOBALS[__FUNCTION__] = ((isConfigEntrySet('DEBUG_MODE')) && (getConfig('DEBUG_MODE') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__];
}

// Checks wether SQL debugging is enabled
function isSqlDebuggingEnabled () {
	// Is cache set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine if SQL debugging is enabled
		$GLOBALS[__FUNCTION__] = ((isConfigEntrySet('DEBUG_SQL')) && (getConfig('DEBUG_SQL') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__];
}

// Checks wether we shall debug regular expressions
function isDebugRegularExpressionEnabled () {
	// Is cache set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Simply check it
		$GLOBALS[__FUNCTION__] = ((isConfigEntrySet('DEBUG_REGEX')) && (getConfig('DEBUG_REGEX') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__];
}

// Checks wether the cache instance is valid
function isCacheInstanceValid () {
	return ((isset($GLOBALS['cache_instance'])) && (is_object($GLOBALS['cache_instance'])));
}

// Copies a file from source to destination and verifies if that goes fine.
// This function should wrap the copy() command and make a nicer debug backtrace
// even if there is no xdebug extension installed.
function copyFileVerified ($source, $dest, $chmod = '') {
	// Failed is the default
	$status = false;

	// Is the source file there?
	if (!isFileReadable($source)) {
		// Then abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot read from source file ' . basename($source) . '.');
	} // END - if

	// Is the target directory there?
	if (!isDirectory(dirname($dest))) {
		// Then abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot find directory ' . str_replace(getPath(), '', dirname($dest)) . '.');
	} // END - if

	// Now try to copy it
	if (!copy($source, $dest)) {
		// Something went wrong
		debug_report_bug(__FUNCTION__, __LINE__, 'copy() has failed to copy the file.');
	} else {
		// Reset cache
		$GLOBALS['file_readable'][$dest] = true;
	}

	// If there are chmod rights set, apply them
	if (!empty($chmod)) {
		// Try to apply them
		$status = changeMode($dest, $chmod);
	} else {
		// All fine
		$status = true;
	}

	// All fine
	return $status;
}

// Wrapper function for header()
// Send a header but checks before if we can do so
function sendHeader ($header) {
	// Send the header
	//* DEBUG: */ logDebugMessage(__FUNCTION__ . ': header=' . $header);
	$GLOBALS['header'][] = trim($header);
}

// Flushes all headers
function flushHeaders () {
	// Is the header already sent?
	if (headers_sent()) {
		// Then abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Headers already sent!');
	} // END - if

	// Flush all headers if found
	if ((isset($GLOBALS['header'])) && (is_array($GLOBALS['header']))) {
		foreach ($GLOBALS['header'] as $header) {
			header($header);
		} // END - foreach
	} // END - if

	// Mark them as flushed
	$GLOBALS['header'] = array();
}

// Wrapper function for chmod()
// @TODO Do some more sanity check here
function changeMode ($FQFN, $mode) {
	// Is the file/directory there?
	if ((!isFileReadable($FQFN)) && (!isDirectory($FQFN))) {
		// Neither, so abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot chmod() on ' . basename($FQFN) . '.');
	} // END - if

	// Try to set them
	return chmod($FQFN, $mode);
}

// Wrapper for unlink()
function removeFile ($FQFN) {
	// Is the file there?
	if (isFileReadable($FQFN)) {
		// Reset cache first
		$GLOBALS['file_readable'][$FQFN] = false;

		// Yes, so remove it
		return unlink($FQFN);
	} // END - if

	// All fine if no file was removed. If we change this to 'false' or rewrite
	// above if() block it would be to restrictive.
	return true;
}

// Wrapper for $_POST['sel']
function countPostSelection ($element = 'sel') {
	// Is it set?
	if (isPostRequestParameterSet($element)) {
		// Return counted elements
		return countSelection(postRequestParameter($element));
	} else {
		// Return zero if not found
		return 0;
	}
}

// Checks wether the config-local.php is loaded
function isConfigLocalLoaded () {
	return ((isset($GLOBALS['config_local_loaded'])) && ($GLOBALS['config_local_loaded'] === true));
}

// Checks wether a nickname or userid was entered and caches the result
function isNicknameUsed ($userid) {
	// Is the cache there
	if (!isset($GLOBALS[__FUNCTION__][$userid])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$userid] = (('' . round($userid) . '') != $userid);
	} // END - if

	// Return the result
	return $GLOBALS[__FUNCTION__][$userid];
}

// Getter for 'what' value
function getWhat () {
	// Default is null
	$what = null;

	// Is the value set?
	if (isWhatSet(true)) {
		// Then use it
		$what = $GLOBALS['what'];
	} // END - if

	// Return it
	return $what;
}

// Setter for 'what' value
function setWhat ($newWhat) {
	$GLOBALS['what'] = SQL_ESCAPE($newWhat);
}

// Setter for 'what' from configuration
function setWhatFromConfig ($configEntry) {
	// Get 'what' from config
	$what = getConfig($configEntry);

	// Set it
	setWhat($what);
}

// Checks wether what is set and optionally aborts on miss
function isWhatSet ($strict =  false) {
	// Check for it
	$isset = isset($GLOBALS['what']);

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'what is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'action' value
function getAction ($strict = true) {
	// Default is null
	$action = null;

	// Is the value set?
	if (isActionSet(($strict) && (isHtmlOutputMode()))) {
		// Then use it
		$action = $GLOBALS['action'];
	} // END - if

	// Return it
	return $action;
}

// Setter for 'action' value
function setAction ($newAction) {
	$GLOBALS['action'] = SQL_ESCAPE($newAction);
}

// Checks wether action is set and optionally aborts on miss
function isActionSet ($strict =  false) {
	// Check for it
	$isset = ((isset($GLOBALS['action'])) && (!empty($GLOBALS['action'])));

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'action is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'module' value
function getModule ($strict = true) {
	// Default is null
	$module = null;

	// Is the value set?
	if (isModuleSet($strict)) {
		// Then use it
		$module = $GLOBALS['module'];
	} // END - if

	// Return it
	return $module;
}

// Setter for 'module' value
function setModule ($newModule) {
	// Secure it and make all modules lower-case
	$GLOBALS['module'] = SQL_ESCAPE(strtolower($newModule));
}

// Checks wether module is set and optionally aborts on miss
function isModuleSet ($strict =  false) {
	// Check for it
	$isset = (!empty($GLOBALS['module']));

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'module is empty.');
	} // END - if

	// Return it
	return (($isset === true) && ($GLOBALS['module'] != 'unknown')) ;
}

// Getter for 'output_mode' value
function getScriptOutputMode () {
	// Default is null
	$output_mode = null;

	// Is the value set?
	if (isOutputModeSet(true)) {
		// Then use it
		$output_mode = $GLOBALS['output_mode'];
	} // END - if

	// Return it
	return $output_mode;
}

// Setter for 'output_mode' value
function setOutputMode ($newOutputMode) {
	$GLOBALS['output_mode'] = (int) $newOutputMode;
}

// Checks wether output_mode is set and optionally aborts on miss
function isOutputModeSet ($strict =  false) {
	// Check for it
	$isset = (isset($GLOBALS['output_mode']));

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'Output_mode is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Enables block-mode
function enableBlockMode ($enabled = true) {
	$GLOBALS['block_mode'] = $enabled;
}

// Checks wether block-mode is enabled
function isBlockModeEnabled () {
	// Abort if not set
	if (!isset($GLOBALS['block_mode'])) {
		// Needs to be fixed
		debug_report_bug(__FUNCTION__, __LINE__, 'Block_mode is not set.');
	} // END - if

	// Return it
	return $GLOBALS['block_mode'];
}

// Wrapper function for addPointsThroughReferalSystem()
function addPointsDirectly ($subject, $userid, $points) {
	// Reset level here
	unset($GLOBALS['ref_level']);

	// Call more complicated method (due to more parameters)
	return addPointsThroughReferalSystem($subject, $userid, $points, false, 0, 'direct');
}

// Wrapper for redirectToUrl but URL comes from a configuration entry
function redirectToConfiguredUrl ($configEntry) {
	// Load the URL
	redirectToUrl(getConfig($configEntry));
}

// Wrapper function to redirect from member-only modules to index
function redirectToIndexMemberOnlyModule () {
	// Do the redirect here
	redirectToUrl('modules.php?module=index&code=' . getCode('MODULE_MEMBER_ONLY') . '&mod=' . getModule());
}

// Wrapper function to redirect to current URL
function redirectToRequestUri () {
	redirectToUrl(basename(detectRequestUri()));
}

// Wrapper function to redirect to de-refered URL
function redirectToDereferedUrl ($URL) {
	// Redirect to to
	redirectToUrl(generateDerefererUrl($URL));
}

// Wrapper function for checking if extension is installed and newer or same version
function isExtensionInstalledAndNewer ($ext_name, $version) {
	// Is an cache entry found?
	if (!isset($GLOBALS[__FUNCTION__][$ext_name][$version])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$ext_name][$version] = ((isExtensionInstalled($ext_name)) && (getExtensionVersion($ext_name) >= $version));
	} else {
		// Cache hits should be incremented twice
		incrementStatsEntry('cache_hits', 2);
	}

	// Return it
	//* DEBUG: */ debugOutput(__FUNCTION__.':'.$ext_name.'=&gt;'.$version.':'.intval($GLOBALS[__FUNCTION__][$ext_name][$version]));
	return $GLOBALS[__FUNCTION__][$ext_name][$version];
}

// Wrapper function for checking if extension is installed and older than given version
function isExtensionInstalledAndOlder ($ext_name, $version) {
	// Is an cache entry found?
	if (!isset($GLOBALS[__FUNCTION__][$ext_name][$version])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$ext_name][$version] = ((isExtensionInstalled($ext_name)) && (isExtensionOlder($ext_name, $version)));
	} else {
		// Cache hits should be incremented twice
		incrementStatsEntry('cache_hits', 2);
	}

	// Return it
	//* DEBUG: */ debugOutput(__FUNCTION__.':'.$ext_name.'&lt;'.$version.':'.intval($GLOBALS[__FUNCTION__][$ext_name][$version]));
	return $GLOBALS[__FUNCTION__][$ext_name][$version];
}

// Set username
function setUsername ($userName) {
	$GLOBALS['username'] = (string) $userName;
}

// Get username
function getUsername () {
	// User name set?
	if (!isset($GLOBALS['username'])) {
		// No, so it has to be a guest
		$GLOBALS['username'] = '{--USERNAME_GUEST--}';
	} // END - if

	// Return it
	return $GLOBALS['username'];
}

// Wrapper function for installation phase
function isInstallationPhase () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((!isInstalled()) || (isInstalling()));
	} // END - if

	// Return result
	return $GLOBALS[__FUNCTION__];
}

// Checks wether the extension demo is actuve and the admin login is demo (password needs to be demo, too!)
function isDemoModeActive () {
	// Is cache set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Simply check it
		$GLOBALS[__FUNCTION__] = ((isExtensionActive('demo')) && (getCurrentAdminLogin() == 'demo'));
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__];
}

// Getter for PHP caching value
function getPhpCaching () {
	return $GLOBALS['php_caching'];
}

// Checks wether the admin hash is set
function isAdminHashSet ($adminId) {
	// Is the array there?
	if (!isset($GLOBALS['cache_array']['admin'])) {
		// Missing array should be reported
		debug_report_bug(__FUNCTION__, __LINE__, 'Cache not set.');
	} // END - if

	// Check for admin hash
	return isset($GLOBALS['cache_array']['admin']['password'][$adminId]);
}

// Setter for admin hash
function setAdminHash ($adminId, $hash) {
	$GLOBALS['cache_array']['admin']['password'][$adminId] = $hash;
}

// Getter for current admin login
function getCurrentAdminLogin () {
	// Log debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'called!');

	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getAdminLogin(getCurrentAdminId());
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__];
}

// Setter for admin id (and current)
function setAdminId ($adminId) {
	// Log debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'adminId=' . $adminId);

	// Set session
	$status = setSession('admin_id', bigintval($adminId));

	// Set current id
	setCurrentAdminId($adminId);

	// Return status
	return $status;
}

// Setter for admin_last
function setAdminLast ($adminLast) {
	// Log debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'adminLast=' . $adminLast);

	// Set session
	$status = setSession('admin_last', $adminLast);

	// Return status
	return $status;
}

// Setter for admin_md5
function setAdminMd5 ($adminMd5) {
	// Log debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'adminMd5=' . $adminMd5);

	// Set session
	$status = setSession('admin_md5', $adminMd5);

	// Return status
	return $status;
}

// Getter for admin_md5
function getAdminMd5 () {
	// Log debug message
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'called!');

	// Get session
	return getSession('admin_md5');
}

// Init user data array
function initUserData () {
	// User id should not be zero
	if (!isValidUserId(getCurrentUserId())) {
		// Should be always valid
		debug_report_bug(__FUNCTION__, __LINE__, 'Current user id is invalid: ' . getCurrentUserId());
	} // END - if

	// Init the user
	$GLOBALS['user_data'][getCurrentUserId()] = array();
}

// Getter for user data
function getUserData ($column) {
	// User id should not be zero
	if (!isValidUserId(getCurrentUserId())) {
		// Should be always valid
		debug_report_bug(__FUNCTION__, __LINE__, 'Current user id is invalid: ' . getCurrentUserId());
	} // END - if

	// Return the value
	return $GLOBALS['user_data'][getCurrentUserId()][$column];
}

// Geter for whole user data array
function getUserDataArray () {
	// Get user id
	$userid = getCurrentUserId();

	// Is the current userid valid?
	if (!isValidUserId($userid)) {
		// Should be always valid
		debug_report_bug(__FUNCTION__, __LINE__, 'Current user id is invalid: ' . $userid);
	} // END - if

	// Get the whole array if found
	if (isset($GLOBALS['user_data'][$userid])) {
		// Found, so return it
		return $GLOBALS['user_data'][$userid];
	} else {
		// Return empty array
		return array();
	}
}

// Checks if the user data is valid, this may indicate that the user has logged
// in, but you should use isMember() if you want to find that out.
function isUserDataValid () {
	// User id should not be zero so abort here
	if (!isCurrentUserIdSet()) return false;

	// Is it cached?
	if (!isset($GLOBALS['is_userdata_valid'][getCurrentUserId()])) {
		// Determine it
		$GLOBALS['is_userdata_valid'][getCurrentUserId()] = ((isset($GLOBALS['user_data'][getCurrentUserId()])) && (count($GLOBALS['user_data'][getCurrentUserId()]) > 1));
	} // END - if

	// Return the result
	return $GLOBALS['is_userdata_valid'][getCurrentUserId()];
}

// Setter for current userid
function setCurrentUserId ($userid) {
	// Set userid
	$GLOBALS['current_userid'] = bigintval($userid);

	// Unset it to re-determine the actual state
	unset($GLOBALS['is_userdata_valid'][$userid]);
}

// Getter for current userid
function getCurrentUserId () {
	// Userid must be set before it can be used
	if (!isCurrentUserIdSet()) {
		// Not set
		debug_report_bug(__FUNCTION__, __LINE__, 'User id is not set.');
	} // END - if

	// Return the userid
	return $GLOBALS['current_userid'];
}

// Checks if current userid is set
function isCurrentUserIdSet () {
	return ((isset($GLOBALS['current_userid'])) && (isValidUserId($GLOBALS['current_userid'])));
}

// Checks wether we are debugging template cache
function isDebuggingTemplateCache () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('DEBUG_TEMPLATE_CACHE') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Wrapper for fetchUserData() and getUserData() calls
function getFetchedUserData ($keyColumn, $userid, $valueColumn) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$userid][$keyColumn][$valueColumn])) {
		// Default is 'guest'
		$data = '{--USERNAME_GUEST--}';

		// Can we fetch the user data?
		if ((isValidUserId($userid)) && (fetchUserData($userid, $keyColumn))) {
			// Now get the data back
			$data = getUserData($valueColumn);
		} // END - if

		// Cache it
		$GLOBALS[__FUNCTION__][$userid][$keyColumn][$valueColumn] = $data;
	} // END - if

	// Return it
	return $GLOBALS[__FUNCTION__][$userid][$keyColumn][$valueColumn];
}

// Wrapper for strpos() to ease porting from deprecated ereg() function
function isInString ($needle, $haystack) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'needle=' . $needle . ', haystack=' . $haystack . ', returned=' . intval(strpos($haystack, $needle) !== false));
	return (strpos($haystack, $needle) !== false);
}

// Wrapper for strpos() to ease porting from deprecated eregi() function
// This function is case-insensitive
function isInStringIgnoreCase ($needle, $haystack) {
	return (isInString(strtolower($needle), strtolower($haystack)));
}

// Wrapper to check for if fatal errors where detected
function ifFatalErrorsDetected () {
	// Just call the inner function
	return (getTotalFatalErrors() > 0);
}

// Setter for HTTP status
function setHttpStatus ($status) {
	$GLOBALS['http_status'] = (string) $status;
}

// Getter for HTTP status
function getHttpStatus () {
	// Is the status set?
	if (!isset($GLOBALS['http_status'])) {
		// Abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'No HTTP status set!');
	} // END - if

	// Return it
	return $GLOBALS['http_status'];
}

/**
 * Send a HTTP redirect to the browser. This function was taken from DokuWiki
 * (GNU GPL 2; http://www.dokuwiki.org) and modified to fit into mailer project.
 *
 * ----------------------------------------------------------------------------
 * If you want to redirect, please use redirectToUrl(); instead
 * ----------------------------------------------------------------------------
 *
 * Works arround Microsoft IIS cookie sending bug. Does exit the script.
 *
 * @link    http://support.microsoft.com/kb/q176113/
 * @author  Andreas Gohr <andi@splitbrain.org>
 * @access  private
 */
function sendRawRedirect ($url) {
	// Send helping header
	setHttpStatus('302 Found');

	// always close the session
	session_write_close();

	// Revert entity &amp;
	$url = str_replace('&amp;', '&', $url);

	// check if running on IIS < 6 with CGI-PHP
	if ((isset($_SERVER['SERVER_SOFTWARE'])) && (isset($_SERVER['GATEWAY_INTERFACE'])) &&
		(strpos($_SERVER['GATEWAY_INTERFACE'], 'CGI') !== false) &&
		(preg_match('|^Microsoft-IIS/(\d)\.\d$|', trim($_SERVER['SERVER_SOFTWARE']), $matches)) &&
		($matches[1] < 6)) {
		// Send the IIS header
		sendHeader('Refresh: 0;url=' . $url);
	} else {
		// Send generic header
		sendHeader('Location: ' . $url);
	}

	// Shutdown here
	shutdown();
}

// Determines the country of the given user id
function determineCountry ($userid) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$userid])) {
		// Default is 'invalid'
		$GLOBALS[__FUNCTION__][$userid] = 'invalid';

		// Is extension country active?
		if (isExtensionActive('country')) {
			// Determine the right country code through the country id
			$id = getUserData('country_code');

			// Then handle it over
			$GLOBALS[__FUNCTION__][$userid] = generateCountryInfo($id);
		} else {
			// Get raw code from user data
			$GLOBALS[__FUNCTION__][$userid] = getUserData('country');
		}
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$userid];
}

// "Getter" for total confirmed user accounts
function getTotalConfirmedUser () {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Then do it
		$GLOBALS[__FUNCTION__] = countSumTotalData('CONFIRMED', 'user_data', 'userid', 'status', true);
	} // END - if

	// Return cached value
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for total unconfirmed user accounts
function getTotalUnconfirmedUser () {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Then do it
		$GLOBALS[__FUNCTION__] = countSumTotalData('UNCONFIRMED', 'user_data', 'userid', 'status', true);
	} // END - if

	// Return cached value
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for total locked user accounts
function getTotalLockedUser () {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Then do it
		$GLOBALS[__FUNCTION__] = countSumTotalData('LOCKED', 'user_data', 'userid', 'status', true);
	} // END - if

	// Return cached value
	return $GLOBALS[__FUNCTION__];
}

// Is given userid valid?
function isValidUserId ($userid) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$userid])) {
		// Check it out
		$GLOBALS[__FUNCTION__][$userid] = ((!is_null($userid)) && (!empty($userid)) && ($userid > 0));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$userid];
}

// Encodes entities
function encodeEntities ($str) {
	// Secure it first
	$str = secureString($str, true, true);

	// Encode dollar sign as well
	$str = str_replace('$', '&#36;', $str);

	// Return it
	return $str;
}

// "Getter" for date from patch_ctime
function getDateFromPatchTime () {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Then set it
		$GLOBALS[__FUNCTION__] = generateDateTime(getConfig('patch_ctime'), '5');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Getter for current year (default)
function getYear ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('Y', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current month (default)
function getMonth ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// If null is set, use time()
		if (is_null($timestamp)) {
			// Use time() which is current timestamp
			$timestamp = time();
		} // END - if

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('m', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current day (default)
function getDay ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('d', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current week (default)
function getWeek ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('W', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current short_hour (default)
function getShortHour ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('G', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current long_hour (default)
function getLongHour ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('H', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current second (default)
function getSecond ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('s', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Getter for current minute (default)
function getMinute ($timestamp = null) {
	// Is it cached?
	if (!isset($GLOBALS[__FUNCTION__][$timestamp])) {
		// null is time()
		if (is_null($timestamp)) $timestamp = time();

		// Then create it
		$GLOBALS[__FUNCTION__][$timestamp] = date('i', $timestamp);
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$timestamp];
}

// Checks wether the title decoration is enabled
function isTitleDecorationEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Just check it
		$GLOBALS[__FUNCTION__] = (getConfig('enable_title_deco') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether filter usage updates are enabled (expensive queries!)
function isFilterUsageUpdateEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isExtensionInstalledAndNewer('sql_patches', '0.6.0')) && (isConfigEntrySet('update_filter_usage')) && (getConfig('update_filter_usage') == 'Y'));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether debugging of weekly resets is enabled
function isWeeklyResetDebugEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isConfigEntrySet('DEBUG_WEEKLY')) && (getConfig('DEBUG_WEEKLY') == 'Y'));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether debugging of monthly resets is enabled
function isMonthlyResetDebugEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isConfigEntrySet('DEBUG_MONTHLY')) && (getConfig('DEBUG_MONTHLY') == 'Y'));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether displaying of debug SQLs are enabled
function isDisplayDebugSqlEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isExtensionInstalledAndNewer('other', '0.2.2')) && (getConfig('display_debug_sqls') == 'Y'));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether module title is enabled
function isModuleTitleEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('enable_mod_title') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether what title is enabled
function isWhatTitleEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('enable_what_title') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether stats are enabled
function ifStatsAreEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Then determine it
		$GLOBALS[__FUNCTION__] = (getConfig('stats_enabled') == 'Y');
	} // END - if

	// Return cached value
	return $GLOBALS[__FUNCTION__];
}

// Checks wether admin-notification of certain user actions is enabled
function isAdminNotificationEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('admin_notify') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether random referal id selection is enabled
function isRandomReferalIdEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isExtensionInstalledAndNewer('user', '0.3.4')) && (getConfig('select_user_zero_refid') == 'Y'));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for default language
function getDefaultLanguage () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('DEFAULT_LANG');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for default referal id
function getDefRefid () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('def_refid');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for path
function getPath () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('PATH');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for url
function getUrl () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('URL');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for cache_path
function getCachePath () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('CACHE_PATH');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for secret_key
function getSecretKey () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('secret_key');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for master_salt
function getMasterSalt () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('master_salt');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for prime
function getPrime () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('_PRIME');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for encrypt_seperator
function getEncryptSeperator () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ENCRYPT_SEPERATOR');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for mysql_prefix
function getMysqlPrefix () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('_MYSQL_PREFIX');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for table_type
function getTableType () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('_TABLE_TYPE');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for salt_length
function getSaltLength () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('salt_length');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for output_mode
function getOutputMode () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('OUTPUT_MODE');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for full_version
function getFullVersion () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('FULL_VERSION');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for title
function getTitle () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('TITLE');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for curr_svn_revision
function getCurrentRepositoryRevision () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('CURRENT_REPOSITORY_REVISION');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for server_url
function getServerUrl () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('SERVER_URL');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for mt_word
function getMtWord () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('mt_word');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for mt_word2
function getMtWord2 () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('mt_word2');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for main_title
function getMainTitle () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('MAIN_TITLE');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for file_hash
function getFileHash () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('file_hash');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for pass_scramble
function getPassScramble () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('pass_scramble');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_inactive_since
function getApInactiveSince () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_inactive_since');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for user_min_confirmed
function getUserMinConfirmed () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('user_min_confirmed');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for auto_purge
function getAutoPurge () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('auto_purge');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for bonus_userid
function getBonusUserid () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('bonus_userid');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_inactive_time
function getApInactiveTime () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_inactive_time');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_dm_timeout
function getApDmTimeout () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_dm_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_tasks_time
function getApTasksTime () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_tasks_time');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ap_unconfirmed_time
function getApUnconfirmedTime () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ap_unconfirmed_time');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for points
function getPoints () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('POINTS');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for slogan
function getSlogan () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('SLOGAN');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for copy
function getCopy () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('COPY');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for webmaster
function getWebmaster () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('WEBMASTER');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for sql_count
function getSqlCount () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('sql_count');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for num_templates
function getNumTemplates () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('num_templates');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for dns_cache_timeout
function getDnsCacheTimeout () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('dns_cache_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for menu_blur_spacer
function getMenuBlurSpacer () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('menu_blur_spacer');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for points_register
function getPointsRegister () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('points_register');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for points_ref
function getPointsRef () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('points_ref');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for ref_payout
function getRefPayout () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ref_payout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for online_timeout
function getOnlineTimeout () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('online_timeout');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for index_home
function getIndexHome () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('index_home');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for one_day
function getOneDay () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('ONE_DAY');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for activate_xchange
function getActivateXchange () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('activate_xchange');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for img_type
function getImgType () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('img_type');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for code_length
function getCodeLength () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('code_length');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for least_cats
function getLeastCats () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('least_cats');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for pass_len
function getPassLen () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('pass_len');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for admin_menu
function getAdminMenu () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('admin_menu');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for last_month
function getLastMonth () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('last_month');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for max_send
function getMaxSend () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('max_send');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for mails_page
function getMailsPage () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('mails_page');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for __DB_NAME
function getDbName () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = getConfig('__DB_NAME');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether proxy configuration is used
function isProxyUsed () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isExtensionInstalledAndNewer('sql_patches', '0.4.3')) && (getConfig('proxy_host') != '') && (getConfig('proxy_port') > 0));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether POST data contains selections
function ifPostContainsSelections ($element = 'sel') {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$element])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$element] = ((isPostRequestParameterSet($element)) && (countPostSelection($element) > 0));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$element];
}

// Checks wether verbose_sql is Y and returns true/false if so
function isVerboseSqlEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = ((isExtensionInstalledAndNewer('sql_patches', '0.0.7')) && (getConfig('verbose_sql') == 'Y'));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// "Getter" for total user points
function getTotalPoints ($userid) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$userid])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$userid] = countSumTotalData($userid, 'user_points', 'points') - countSumTotalData($userid, 'user_data', 'used_points');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$userid];
}

// Wrapper to check if url_blacklist is enabled
function isUrlBlacklistEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('url_blacklist') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Checks wether direct payment is allowed in configuration
function isDirectPaymentEnabled () {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Determine it
		$GLOBALS[__FUNCTION__] = (getConfig('allow_direct_pay') == 'Y');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// Wrapper to check if current task is for extension (not update)
function isExtensionTask ($content) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$content['task_type'] . '_' . $content['infos']])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$content['task_type'] . '_' . $content['infos']] = (($content['task_type'] == 'EXTENSION') && (isExtensionNameValid($content['infos'])) && (!isExtensionInstalled($content['infos'])));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$content['task_type'] . '_' . $content['infos']];
}

// Wrapper to check if output mode is CSS
function isCssOutputMode () {
	// Determine it
	return (getScriptOutputMode() == 1);
}

// Wrapper to check if output mode is HTML
function isHtmlOutputMode () {
	// Determine it
	return (getScriptOutputMode() == 0);
}

// Wrapper to check if output mode is RAW
function isRawOutputMode () {
	// Determine it
	return (getScriptOutputMode() == -1);
}

// Wrapper to generate a user email link
function generateWrappedUserEmailLink ($email) {
	// Just call the inner function
	return generateEmailLink($email, 'user_data');
}

// Wrapper to check if user points are locked
function ifUserPointsLocked ($userid) {
	// Do we have cache?
	if (!isset($GLOBALS[__FUNCTION__][$userid])) {
		// Determine it
		$GLOBALS[__FUNCTION__][$userid] = ((getFetchedUserData('userid', $userid, 'ref_payout') > 0) && (!isDirectPaymentEnabled()));
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__][$userid];
}

// Appends a line to an existing file or creates it instantly with given content.
// This function does always add a new-line character to every line.
function appendLineToFile ($file, $line) {
	$fp = fopen($file, 'a') or debug_report_bug(__FUNCTION__, __LINE__, 'Cannot write to file ' . basename($file) . '!');
	fwrite($fp, $line . "\n");
	fclose($fp);
}

// Wrapper for changeDataInFile() but with full path added
function changeDataInInclude ($FQFN, $comment, $prefix, $suffix, $DATA, $seek=0) {
	// Add full path
	$FQFN = getPath() . $FQFN;

	// Call inner function
	changeDataInFile($FQFN, $comment, $prefix, $suffix, $DATA, $seek);
}

// Shortens ucfirst(strtolower()) calls
function firstCharUpperCase ($str) {
	return ucfirst(strtolower($str));
}

// [EOF]
?>
